-- =============================================
-- SaveToDB Framework for Microsoft SQL Server
-- Version 5.0, March 12, 2018
--
-- This script upgrades SaveToDB Framework 3.x-4.x to SaveToDB Framework 5.0
--
-- Copyright 2011-2018 Gartle Technology Corporation
-- =============================================
GO

BEGIN TRANSACTION

-- ===== EventHandlers =====

IF EXISTS(
SELECT
    c.TABLE_SCHEMA
    , c.TABLE_NAME
    , COUNT(*) AS FieldCount
FROM
    INFORMATION_SCHEMA.COLUMNS c
    INNER JOIN INFORMATION_SCHEMA.TABLES t ON t.TABLE_SCHEMA = c.TABLE_SCHEMA AND t.TABLE_NAME = c.TABLE_NAME
WHERE
    (
        UPPER(c.COLUMN_NAME) = 'ID'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_SCHEMA'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_NAME'
        OR UPPER(c.COLUMN_NAME) = 'COLUMN_NAME'
        OR UPPER(c.COLUMN_NAME) = 'EVENT_NAME'
        OR UPPER(c.COLUMN_NAME) = 'HANDLER_SCHEMA'
        OR UPPER(c.COLUMN_NAME) = 'HANDLER_NAME'
        OR UPPER(c.COLUMN_NAME) = 'HANDLER_TYPE'
        OR UPPER(c.COLUMN_NAME) = 'HANDLER_CODE'
        OR UPPER(c.COLUMN_NAME) = 'TARGET_WORKSHEET'
        OR UPPER(c.COLUMN_NAME) = 'MENU_ORDER'
        OR UPPER(c.COLUMN_NAME) = 'EDIT_PARAMETERS'
        OR UPPER(c.COLUMN_NAME) = 'IS_ACTIVE'
    ) AND t.TABLE_SCHEMA = 'dbo01'
    AND t.TABLE_NAME = 'EventHandlers'
GROUP BY
    c.TABLE_SCHEMA
    , c.TABLE_NAME
HAVING
    COUNT(*) = 12
    AND MAX(c.ORDINAL_POSITION) = 12
)
BEGIN TRY
-- Creating a temporary EventHandlers table

IF OBJECT_ID('dbo01.Tmp_EventHandlers', 'U') IS NOT NULL
    DROP TABLE dbo01.Tmp_EventHandlers

CREATE TABLE [dbo01].[Tmp_EventHandlers] (
    [ID] [int] IDENTITY(1,1) NOT NULL,
    [TABLE_SCHEMA] [nvarchar](128) NOT NULL,
    [TABLE_NAME] [nvarchar](128) NOT NULL,
    [COLUMN_NAME] [nvarchar](128),
    [EVENT_NAME] [varchar](50) NOT NULL,
    [HANDLER_SCHEMA] [nvarchar](128),
    [HANDLER_NAME] [nvarchar](128) NOT NULL,
    [HANDLER_TYPE] [nvarchar](128),
    [HANDLER_CODE] [nvarchar](max),
    [TARGET_WORKSHEET] [nvarchar](128),
    [MENU_ORDER] [int],
    [EDIT_PARAMETERS] [bit],
    [IS_ACTIVE] [bit],
  CONSTRAINT [PK_Tmp_EventHandlers_dbo01] PRIMARY KEY ([ID])
)

-- Copying existing handlers

SET IDENTITY_INSERT dbo01.Tmp_EventHandlers ON

INSERT INTO dbo01.Tmp_EventHandlers
    (ID, TABLE_SCHEMA, TABLE_NAME, COLUMN_NAME, EVENT_NAME, HANDLER_SCHEMA, HANDLER_NAME, HANDLER_TYPE, HANDLER_CODE, TARGET_WORKSHEET, MENU_ORDER, EDIT_PARAMETERS, IS_ACTIVE)
SELECT
    c.ID
    , c.TABLE_SCHEMA
    , c.TABLE_NAME
    , c.COLUMN_NAME
    , c.EVENT_NAME
    , CASE
        WHEN t.TABLE_TYPE IS NOT NULL THEN t.TABLE_SCHEMA
        WHEN t1.TABLE_TYPE IS NOT NULL THEN t1.TABLE_SCHEMA
        WHEN r.ROUTINE_TYPE IS NOT NULL THEN r.ROUTINE_SCHEMA
        WHEN r1.ROUTINE_TYPE IS NOT NULL THEN r1.ROUTINE_SCHEMA
        WHEN c.HANDLER_TYPE IN ('HTTP', 'TEXT', 'CODE', 'MACRO', 'CMD') THEN c.HANDLER_TYPE
        WHEN CHARINDEX('=http://', c.HANDLER_NAME) > 1 THEN 'HTTP'
        WHEN CHARINDEX('=https://', c.HANDLER_NAME) > 1 THEN 'HTTP'
        WHEN CHARINDEX('http://', c.HANDLER_NAME) = 1 THEN 'HTTP'
        WHEN CHARINDEX('https://', c.HANDLER_NAME) = 1 THEN 'HTTP'
        ELSE c.HANDLER_SCHEMA END
        AS HANDLER_SCHEMA
    , CASE
        WHEN t.TABLE_TYPE IS NOT NULL THEN t.TABLE_NAME
        WHEN t1.TABLE_TYPE IS NOT NULL THEN t1.TABLE_NAME
        WHEN r.ROUTINE_TYPE IS NOT NULL THEN r.ROUTINE_NAME
        WHEN r1.ROUTINE_TYPE IS NOT NULL THEN r1.ROUTINE_NAME
        WHEN CHARINDEX('=http://', c.HANDLER_NAME) > 1  THEN SUBSTRING(c.HANDLER_NAME, 1, CHARINDEX('=http://', c.HANDLER_NAME) - 1)
        WHEN CHARINDEX('=https://', c.HANDLER_NAME) > 1 THEN SUBSTRING(c.HANDLER_NAME, 1, CHARINDEX('=https://', c.HANDLER_NAME) - 1)
        ELSE c.HANDLER_NAME END
        AS HANDLER_NAME
    , COALESCE(t.TABLE_TYPE, t1.TABLE_TYPE, r.ROUTINE_TYPE, r1.ROUTINE_TYPE, c.HANDLER_TYPE, CASE
        WHEN CHARINDEX('=http://', c.HANDLER_NAME) > 1 THEN 'HTTP'
        WHEN CHARINDEX('=https://', c.HANDLER_NAME) > 1 THEN 'HTTP'
        WHEN CHARINDEX('http://', c.HANDLER_NAME) = 1 THEN 'HTTP'
        WHEN CHARINDEX('https://', c.HANDLER_NAME) = 1 THEN 'HTTP'
        ELSE NULL
        END ) AS HANDLER_TYPE
    , CASE
        WHEN COALESCE(t.TABLE_TYPE, t1.TABLE_TYPE, r.ROUTINE_TYPE, r1.ROUTINE_TYPE) IS NOT NULL THEN NULL
        WHEN c.HANDLER_TYPE IN ('HTTP', 'TEXT', 'CODE', 'MACRO', 'CMD') AND c.HANDLER_SCHEMA IS NOT NULL THEN c.HANDLER_SCHEMA
        WHEN CHARINDEX('=http://', c.HANDLER_NAME) > 1  THEN SUBSTRING(c.HANDLER_NAME, CHARINDEX('=http://', c.HANDLER_NAME) + 1, 2000)
        WHEN CHARINDEX('=https://', c.HANDLER_NAME) > 1 THEN SUBSTRING(c.HANDLER_NAME, CHARINDEX('=https://', c.HANDLER_NAME) + 1, 2000)
        WHEN CHARINDEX('http://', c.HANDLER_NAME) = 1  THEN c.HANDLER_NAME
        WHEN CHARINDEX('https://', c.HANDLER_NAME) = 1 THEN c.HANDLER_NAME
        ELSE NULL
        END AS HANDLER_CODE
    , c.TARGET_WORKSHEET
    , c.MENU_ORDER
    , c.EDIT_PARAMETERS
    , c.IS_ACTIVE
FROM
    dbo01.EventHandlers c
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.HANDLER_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.HANDLER_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t1 ON UPPER(t1.TABLE_SCHEMA + '.' + t1.TABLE_NAME) = UPPER(c.HANDLER_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.HANDLER_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.HANDLER_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r1 ON UPPER(r1.ROUTINE_SCHEMA + '.' + r1.ROUTINE_NAME) = UPPER(c.HANDLER_NAME)

SET IDENTITY_INSERT dbo01.Tmp_EventHandlers OFF

-- Renaming the previous EventHandlers table

EXECUTE sp_rename N'dbo01.EventHandlers', N'EventHandlersBak', 'OBJECT'

ALTER TABLE dbo01.EventHandlersBak DROP CONSTRAINT DF_EventHandlers_IS_ACTIVE
ALTER TABLE dbo01.EventHandlersBak DROP CONSTRAINT PK_EventHandlers_dbo01

ALTER TABLE dbo01.EventHandlersBak ADD CONSTRAINT PK_EventHandlersBak_dbo01 PRIMARY KEY (ID)
ALTER TABLE dbo01.EventHandlersBak ADD CONSTRAINT DF_EventHandlersBak_IS_ACTIVE  DEFAULT ((1)) FOR IS_ACTIVE

-- Renaming a temporary table

EXECUTE sp_rename N'dbo01.Tmp_EventHandlers', N'EventHandlers', 'OBJECT'

ALTER TABLE dbo01.EventHandlers DROP CONSTRAINT PK_Tmp_EventHandlers_dbo01
ALTER TABLE dbo01.EventHandlers ADD CONSTRAINT PK_EventHandlers_dbo01 PRIMARY KEY (ID)

ALTER TABLE dbo01.EventHandlers ADD CONSTRAINT DF_EventHandlers_IS_ACTIVE  DEFAULT ((1)) FOR IS_ACTIVE

END TRY
BEGIN CATCH
    ROLLBACK
    RETURN
END CATCH
GO

-- ===== ParameterValues =====

IF EXISTS(
SELECT
    c.TABLE_SCHEMA
    , c.TABLE_NAME
    , COUNT(*) AS FieldCount
FROM
    INFORMATION_SCHEMA.COLUMNS c
    INNER JOIN INFORMATION_SCHEMA.TABLES t ON t.TABLE_SCHEMA = t.TABLE_SCHEMA AND t.TABLE_NAME = c.TABLE_NAME
WHERE
    (
        UPPER(c.COLUMN_NAME) = 'ID'
        OR UPPER(c.COLUMN_NAME) = 'SPECIFIC_SCHEMA'
        OR UPPER(c.COLUMN_NAME) = 'SPECIFIC_NAME'
        OR UPPER(c.COLUMN_NAME) = 'PARAMETER_NAME'
        OR UPPER(c.COLUMN_NAME) = 'SELECT_SCHEMA'
        OR UPPER(c.COLUMN_NAME) = 'SELECT_NAME'
        OR UPPER(c.COLUMN_NAME) = 'SELECT_TYPE'
    ) AND t.TABLE_SCHEMA = 'dbo01'
    AND t.TABLE_NAME = 'ParameterValues'
GROUP BY
    c.TABLE_SCHEMA
    , c.TABLE_NAME
HAVING
    COUNT(*) = 7
    AND MAX(c.ORDINAL_POSITION) = 7
)
BEGIN TRY

-- Creating a temporary ParameterValues table

IF OBJECT_ID('dbo01.Tmp_ParameterValues', 'U') IS NOT NULL
    DROP TABLE dbo01.Tmp_ParameterValues

CREATE TABLE [dbo01].[Tmp_ParameterValues] (
    [ID] [int] IDENTITY(1,1) NOT NULL,
    [TABLE_SCHEMA] [nvarchar](128) NOT NULL,
    [TABLE_NAME] [nvarchar](128) NOT NULL,
    [PARAMETER_NAME] [nvarchar](128) NOT NULL,
    [SELECT_SCHEMA] [nvarchar](128),
    [SELECT_NAME] [nvarchar](128) NOT NULL,
    [SELECT_TYPE] [nvarchar](128),
    [SELECT_CODE] [nvarchar](max),
  CONSTRAINT [PK_Tmp_ParameterValues_dbo01] PRIMARY KEY ([ID])
)

-- Copying existing handlers

SET IDENTITY_INSERT dbo01.Tmp_ParameterValues ON

INSERT INTO dbo01.Tmp_ParameterValues
    (ID, TABLE_SCHEMA, TABLE_NAME, PARAMETER_NAME, SELECT_SCHEMA, SELECT_NAME, SELECT_TYPE, SELECT_CODE)
SELECT
    c.ID
    , c.SPECIFIC_SCHEMA
    , c.SPECIFIC_NAME
    , c.PARAMETER_NAME
    , CASE
        WHEN t.TABLE_TYPE IS NOT NULL THEN t.TABLE_SCHEMA
        WHEN t1.TABLE_TYPE IS NOT NULL THEN t1.TABLE_SCHEMA
        WHEN r.ROUTINE_TYPE IS NOT NULL THEN r.ROUTINE_SCHEMA
        WHEN r1.ROUTINE_TYPE IS NOT NULL THEN r1.ROUTINE_SCHEMA
        WHEN c.SELECT_TYPE IN ('CODE') THEN c.SELECT_TYPE
        ELSE c.SELECT_SCHEMA END
        AS SELECT_SCHEMA
    , CASE
        WHEN t.TABLE_TYPE IS NOT NULL THEN t.TABLE_NAME
        WHEN t1.TABLE_TYPE IS NOT NULL THEN t1.TABLE_NAME
        WHEN r.ROUTINE_TYPE IS NOT NULL THEN r.ROUTINE_NAME
        WHEN r1.ROUTINE_TYPE IS NOT NULL THEN r1.ROUTINE_NAME
        ELSE c.SELECT_NAME END
        AS SELECT_NAME
    , COALESCE(t.TABLE_TYPE, t1.TABLE_TYPE, r.ROUTINE_TYPE, r1.ROUTINE_TYPE, c.SELECT_TYPE) AS SELECT_TYPE
    , CASE WHEN c.SELECT_TYPE IN ('CODE') THEN c.SELECT_SCHEMA ELSE NULL END AS SELECT_CODE
FROM
    dbo01.ParameterValues c
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.SELECT_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.SELECT_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t1 ON UPPER(t1.TABLE_SCHEMA + '.' + t1.TABLE_NAME) = UPPER(c.SELECT_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.SELECT_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.SELECT_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r1 ON UPPER(r1.ROUTINE_SCHEMA + '.' + r1.ROUTINE_NAME) = UPPER(c.SELECT_NAME)

SET IDENTITY_INSERT dbo01.Tmp_ParameterValues OFF

-- Renaming the previous ParameterValues table

EXECUTE sp_rename N'dbo01.ParameterValues', N'ParameterValuesBak', 'OBJECT'

ALTER TABLE dbo01.ParameterValuesBak DROP CONSTRAINT PK_ParameterValues_dbo01
ALTER TABLE dbo01.ParameterValuesBak ADD CONSTRAINT PK_ParameterValuesBak_dbo01 PRIMARY KEY (ID)

-- Renaming a temporary table

EXECUTE sp_rename N'dbo01.Tmp_ParameterValues', N'ParameterValues', 'OBJECT'

ALTER TABLE dbo01.ParameterValues DROP CONSTRAINT PK_Tmp_ParameterValues_dbo01
ALTER TABLE dbo01.ParameterValues ADD CONSTRAINT PK_ParameterValues_dbo01 PRIMARY KEY (ID)

CREATE UNIQUE NONCLUSTERED INDEX [IX_ParameterValues_Schema_Name_Parameter_dbo01] ON [dbo01].[ParameterValues]
(
    [TABLE_SCHEMA],
    [TABLE_NAME],
    [PARAMETER_NAME]
)

END TRY
BEGIN CATCH
    ROLLBACK
    RETURN
END CATCH
GO

-- ===== QueryList =====

IF EXISTS(
SELECT
    c.TABLE_SCHEMA
    , c.TABLE_NAME
    , COUNT(*) AS FieldCount
FROM
    INFORMATION_SCHEMA.COLUMNS c
    INNER JOIN INFORMATION_SCHEMA.TABLES t ON t.TABLE_SCHEMA = c.TABLE_SCHEMA AND t.TABLE_NAME = c.TABLE_NAME
WHERE
    (
        UPPER(c.COLUMN_NAME) = 'ID'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_SCHEMA'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_NAME'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_TYPE'
        OR UPPER(c.COLUMN_NAME) = 'TABLE_CODE'
        OR UPPER(c.COLUMN_NAME) = 'INSERT_PROCEDURE'
        OR UPPER(c.COLUMN_NAME) = 'UPDATE_PROCEDURE'
        OR UPPER(c.COLUMN_NAME) = 'DELETE_PROCEDURE'
        OR UPPER(c.COLUMN_NAME) = 'PROCEDURE_TYPE'
    ) AND t.TABLE_SCHEMA = 'dbo01'
    AND t.TABLE_NAME = 'QueryList'
GROUP BY
    c.TABLE_SCHEMA
    , c.TABLE_NAME
HAVING
    COUNT(*) = 7
    AND MAX(c.ORDINAL_POSITION) = 7
)
BEGIN TRY

-- Creating a temporary QueryList table

IF OBJECT_ID('dbo01.Tmp_QueryList', 'U') IS NOT NULL
    DROP TABLE dbo01.Tmp_QueryList

CREATE TABLE [dbo01].[Tmp_QueryList] (
    [ID] [int] IDENTITY(1,1) NOT NULL,
    [TABLE_SCHEMA] [nvarchar](128) NOT NULL,
    [TABLE_NAME] [nvarchar](128) NOT NULL,
    [TABLE_TYPE] [nvarchar](128) NOT NULL,
    [TABLE_CODE] [nvarchar](max),
    [INSERT_PROCEDURE] [nvarchar](max),
    [UPDATE_PROCEDURE] [nvarchar](max),
    [DELETE_PROCEDURE] [nvarchar](max),
    [PROCEDURE_TYPE] [nvarchar](128),
  CONSTRAINT [PK_Tmp_QueryList_dbo01] PRIMARY KEY ([ID])
)

-- Copying existing handlers

SET IDENTITY_INSERT dbo01.Tmp_QueryList ON

INSERT INTO dbo01.Tmp_QueryList
    (ID, TABLE_SCHEMA, TABLE_NAME, TABLE_TYPE, TABLE_CODE, INSERT_PROCEDURE, UPDATE_PROCEDURE, DELETE_PROCEDURE, PROCEDURE_TYPE)
SELECT
    c.ID
    , CASE WHEN c.TABLE_TYPE IN ('CODE', 'HTTP', 'TEXT') THEN c.TABLE_TYPE ELSE c.TABLE_SCHEMA END AS TABLE_SCHEMA
    , c.TABLE_NAME
    , COALESCE(t.TABLE_TYPE, r.ROUTINE_TYPE, c.TABLE_TYPE) AS TABLE_TYPE
    , CASE WHEN c.TABLE_TYPE IN ('CODE', 'HTTP', 'TEXT') THEN c.TABLE_SCHEMA ELSE NULL END AS TABLE_CODE
    , c.INSERT_PROCEDURE
    , c.UPDATE_PROCEDURE
    , c.DELETE_PROCEDURE
    , NULL AS PROCEDURE_TYPE
FROM
    dbo01.QueryList c
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.TABLE_NAME)
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.TABLE_NAME)

SET IDENTITY_INSERT dbo01.Tmp_QueryList OFF

-- Renaming the previous QueryList table

EXECUTE sp_rename N'dbo01.QueryList', N'QueryListBak', 'OBJECT'

ALTER TABLE dbo01.QueryListBak DROP CONSTRAINT PK_QueryList_dbo01
ALTER TABLE dbo01.QueryListBak ADD CONSTRAINT PK_QueryListBak_dbo01 PRIMARY KEY (ID)

-- Renaming a temporary table

EXECUTE sp_rename N'dbo01.Tmp_QueryList', N'QueryList', 'OBJECT'

ALTER TABLE dbo01.QueryList DROP CONSTRAINT PK_Tmp_QueryList_dbo01
ALTER TABLE dbo01.QueryList ADD CONSTRAINT PK_QueryList_dbo01 PRIMARY KEY (ID)

END TRY
BEGIN CATCH
    ROLLBACK
    RETURN
END CATCH
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     5.0, 2015-04-13
-- Description: Configuration view of event handlers
-- =============================================

ALTER VIEW [xls01].[viewEventHandlers]
AS

SELECT
    t.ID
    , t.TABLE_SCHEMA
    , t.TABLE_NAME
    , t.COLUMN_NAME
    , t.EVENT_NAME
    , t.HANDLER_SCHEMA
    , t.HANDLER_NAME
    , t.HANDLER_TYPE
    , t.HANDLER_CODE
    , t.TARGET_WORKSHEET
    , t.MENU_ORDER
    , t.EDIT_PARAMETERS
FROM
    [dbo01].[EventHandlers] t
WHERE
    t.IS_ACTIVE = 1

GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     5.0, 2015-04-13
-- Description: Configuration view of queries of parameter values
-- =============================================

ALTER VIEW [xls01].[viewParameterValues]
AS

SELECT
    t.ID
    , t.TABLE_SCHEMA
    , t.TABLE_NAME
    , t.PARAMETER_NAME
    , t.SELECT_SCHEMA
    , t.SELECT_NAME
    , t.SELECT_TYPE
    , t.SELECT_CODE
FROM
    [dbo01].[ParameterValues] t

GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     5.0, 2015-04-13
-- Description: SaveToDB Query List view
-- =============================================

ALTER VIEW [dbo01].[viewQueryList]
AS

SELECT
    t.TABLE_SCHEMA
    , t.TABLE_NAME
    , t.TABLE_TYPE
    , NULL AS TABLE_CODE
    , COALESCE(q.INSERT_PROCEDURE, i.ROUTINE_SCHEMA + '.' + i.ROUTINE_NAME) AS INSERT_PROCEDURE
    , COALESCE(q.UPDATE_PROCEDURE, u.ROUTINE_SCHEMA + '.' + u.ROUTINE_NAME) AS UPDATE_PROCEDURE
    , COALESCE(q.DELETE_PROCEDURE, d.ROUTINE_SCHEMA + '.' + d.ROUTINE_NAME) AS DELETE_PROCEDURE
    , COALESCE(q.PROCEDURE_TYPE, i.ROUTINE_TYPE) AS PROCEDURE_TYPE
FROM
    (
        SELECT
            t.TABLE_CATALOG
            , t.TABLE_SCHEMA
            , t.TABLE_NAME
            , t.TABLE_TYPE
        FROM
            INFORMATION_SCHEMA.TABLES t
        WHERE
            NOT LEFT(t.TABLE_NAME,3) = 'sys'
            AND HAS_PERMS_BY_NAME(QUOTENAME(t.TABLE_CATALOG)+'.'+QUOTENAME(t.TABLE_SCHEMA)+'.'+QUOTENAME(t.TABLE_NAME), 'OBJECT', 'SELECT') = 1

        UNION
        SELECT
            r.ROUTINE_CATALOG   AS TABLE_CATALOG
            , r.ROUTINE_SCHEMA  AS TABLE_SCHEMA
            , r.ROUTINE_NAME    AS TABLE_NAME
            , r.ROUTINE_TYPE    AS TABLE_TYPE
        FROM 
            INFORMATION_SCHEMA.ROUTINES r
        WHERE
            NOT LEFT(r.ROUTINE_NAME, 3) = 'sp_'
            AND r.ROUTINE_TYPE = 'PROCEDURE'
            AND HAS_PERMS_BY_NAME(QUOTENAME(r.ROUTINE_CATALOG)+'.'+QUOTENAME(r.ROUTINE_SCHEMA)+'.'+QUOTENAME(r.ROUTINE_NAME), 'OBJECT', 'EXECUTE') = 1
            AND NOT r.ROUTINE_SCHEMA IN ('etl01')
            AND NOT r.ROUTINE_NAME LIKE '%_insert'
            AND NOT r.ROUTINE_NAME LIKE '%_update'
            AND NOT r.ROUTINE_NAME LIKE '%_delete'
    ) t

    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES i ON i.ROUTINE_SCHEMA = t.TABLE_SCHEMA AND i.ROUTINE_NAME = REPLACE(t.TABLE_NAME, '_select', '') + '_insert'
        AND HAS_PERMS_BY_NAME(QUOTENAME(i.ROUTINE_CATALOG)+'.'+QUOTENAME(i.ROUTINE_SCHEMA)+'.'+QUOTENAME(i.ROUTINE_NAME), 'OBJECT', 'EXECUTE') = 1
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES u ON u.ROUTINE_SCHEMA = t.TABLE_SCHEMA AND u.ROUTINE_NAME = REPLACE(t.TABLE_NAME, '_select', '') + '_update'
        AND HAS_PERMS_BY_NAME(QUOTENAME(u.ROUTINE_CATALOG)+'.'+QUOTENAME(u.ROUTINE_SCHEMA)+'.'+QUOTENAME(u.ROUTINE_NAME), 'OBJECT', 'EXECUTE') = 1
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES d ON d.ROUTINE_SCHEMA = t.TABLE_SCHEMA AND d.ROUTINE_NAME = REPLACE(t.TABLE_NAME, '_select', '') + '_delete'
        AND HAS_PERMS_BY_NAME(QUOTENAME(d.ROUTINE_CATALOG)+'.'+QUOTENAME(d.ROUTINE_SCHEMA)+'.'+QUOTENAME(d.ROUTINE_NAME), 'OBJECT', 'EXECUTE') = 1
    LEFT OUTER JOIN [dbo01].[QueryList] q ON q.TABLE_SCHEMA = t.TABLE_SCHEMA AND q.TABLE_NAME = t.TABLE_NAME

UNION
SELECT
    q.TABLE_SCHEMA
    , q.TABLE_NAME
    , q.TABLE_TYPE
    , q.TABLE_CODE
    , q.INSERT_PROCEDURE
    , q.UPDATE_PROCEDURE
    , q.DELETE_PROCEDURE
    , q.PROCEDURE_TYPE
FROM
    [dbo01].[QueryList] q
    LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON q.TABLE_SCHEMA = t.TABLE_SCHEMA AND q.TABLE_NAME = t.TABLE_NAME
    LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON q.TABLE_SCHEMA = r.ROUTINE_SCHEMA AND q.TABLE_NAME = r.ROUTINE_NAME
WHERE
    q.TABLE_TYPE IN ('CODE', 'HTTP', 'TEXT')
    AND t.TABLE_NAME IS NULL
    AND r.ROUTINE_NAME IS NULL
GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     5.0, 2015-04-13
-- Description: SaveToDB Query List view
--
-- This view can be used as a data source of other Query List views
-- which can filter different logic areas of a database.
-- =============================================

ALTER VIEW [xls01].[viewQueryList]
AS

SELECT
    t.TABLE_SCHEMA
    , t.TABLE_NAME
    , t.TABLE_TYPE
    , t.TABLE_CODE
    , t.INSERT_PROCEDURE
    , t.UPDATE_PROCEDURE
    , t.DELETE_PROCEDURE
    , t.PROCEDURE_TYPE
FROM
    [dbo01].[viewQueryList] t
WHERE
    NOT t.TABLE_SCHEMA IN ('etl01', 'dbo01')
    AND NOT t.TABLE_NAME IN (                           -- SDK objects are not showed
        'ObjectTranslation'                             -- SDK 3.0
        , 'ColumnTranslation'
        , 'ParameterValues'
        , 'EventHandlers'
        , 'TableFormats'
        , 'QueryList'

        , 'viewQueryList'                               -- SDK 3.0
        , 'viewEventHandlers'
        , 'viewColumnTranslation'
        , 'viewObjectTranslation'
        , 'viewParameterValues'
        , 'viewTableFormats'
        , 'viewObjectTranslationSuggest'
        , 'viewEventHandlersSuggest'

        , 'Translation_Objects'                         -- SDK 2.x
        , 'Translation_Columns'
        , 'ExcelEventHandlers'
        , 'ExcelTableFormats'

        , 'viewTranslation_Columns'
        , 'viewTranslation_Objects'
        , 'viewExcelEventHandlers'
        , 'viewExcelTableFormats'
        , 'viewParameterValuesQueries'

        , 'uspUpdateTableFormat'                        -- SDK 3.0
        , 'uspGenerateEditProcedures'
        , 'uspGenerateChangeHandler'
        , 'uspGenerateGenegicHandler'
        , 'uspUpdateColumnTranslation'
        , 'uspUpdateColumnTranslation_Group'
        , 'uspUpdateObjectTranslation'
        , 'uspUpdateObjectTranslationAbsent'
        , 'uspUpdateTranslation_Columns'
        , 'uspUpdateTranslation_Columns_Group'
        , 'uspUpdateTranslation_Objects'
        , 'uspUpdateEventHandlers'

        , 'uspUpdateExcelTableFormat'                   -- SDK 2.0
        , 'uspUpdateExcelEventHandlers'
        )
    AND NOT t.TABLE_NAME LIKE 'viewParameterValues_%'   -- Parameter values
    AND NOT t.TABLE_NAME LIKE 'uspExcelEvent_%'         -- Event handlers
    AND NOT t.TABLE_NAME LIKE 'uspParameterValues_%'    -- Parameter values

GO

-- =============================================
-- Author:      Gartle Technology Corporation
-- Release:     5.0, 2015-04-13
-- Description: SaveToDB Query List view
-- =============================================

CREATE VIEW [dbo01].[viewQueryListTools]
AS

SELECT
    t.TABLE_SCHEMA
    , t.TABLE_NAME
    , t.TABLE_TYPE
    , t.TABLE_CODE
    , t.INSERT_PROCEDURE
    , t.UPDATE_PROCEDURE
    , t.DELETE_PROCEDURE
    , t.PROCEDURE_TYPE
FROM
    [dbo01].[viewQueryList] t
WHERE
    t.TABLE_SCHEMA IN ('tools01')

GO

INSERT [dbo01].[QueryList] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_TYPE], [TABLE_CODE], [INSERT_PROCEDURE], [UPDATE_PROCEDURE], [DELETE_PROCEDURE], [PROCEDURE_TYPE]) VALUES (N'tools01', N'DatabaseObjects', N'CODE', N'SELECT
    so.[object_id] AS ID
    , QUOTENAME(SCHEMA_NAME(so.[schema_id])) + ''.'' + QUOTENAME(so.name) AS ObjectName
    , SCHEMA_NAME(so.[schema_id]) AS [Schema]
    , so.name AS Name
    , [Type] = CASE so.type_desc
        WHEN ''USER_TABLE'' THEN ''table''
        WHEN ''VIEW'' THEN ''view''
        WHEN ''SQL_STORED_PROCEDURE'' THEN ''procedure''
        WHEN ''SQL_TRIGGER'' THEN ''trigger''
        ELSE so.type_desc
        END
    , t.TRANSLATED_NAME AS [TranslatedName]
    , t.TRANSLATED_DESC AS [TranslatedDescription]
    , t.TRANSLATED_COMMENT AS [TranslatedComment]
    , CASE
        WHEN HAS_PERMS_BY_NAME(QUOTENAME(DB_NAME())+''.''+QUOTENAME(SCHEMA_NAME(so.[schema_id]))+''.''+QUOTENAME(so.name), ''OBJECT'', ''ALTER'') = 1 THEN ''yes''
        ELSE NULL
        END AS Alterable
    , so.create_date AS CreateDate
    , so.modify_date AS ModifyDate
FROM
    sys.objects so
    LEFT OUTER JOIN dbo01.ObjectTranslation t ON t.TABLE_SCHEMA = SCHEMA_NAME(so.[schema_id]) AND t.TABLE_NAME = so.name
        AND t.LANGUAGE_NAME = @Language
WHERE
    NOT SCHEMA_NAME(so.[schema_id]) = ''sys''
    AND (type_desc = ''SQL_STORED_PROCEDURE''
        OR type_desc = ''SQL_TRIGGER''
        OR type_desc = ''USER_TABLE''
        OR type_desc = ''VIEW''
    )
    AND NOT (so.name LIKE ''sp_%'' AND SCHEMA_NAME(so.[schema_id]) = ''dbo'')
    AND so.name NOT LIKE ''sys%''
    AND SCHEMA_NAME(so.[schema_id]) NOT IN (''dbo01'', ''etl01'', ''xls01'')
', NULL, NULL, NULL, NULL)
GO
INSERT [dbo01].[QueryList] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_TYPE], [TABLE_CODE], [INSERT_PROCEDURE], [UPDATE_PROCEDURE], [DELETE_PROCEDURE], [PROCEDURE_TYPE]) VALUES (N'tools01', N'RoleMembers', N'CODE', N'DECLARE @List varchar(MAX)

SELECT @List = (
    SELECT '', ['' + name + '']'' FROM (
        SELECT DISTINCT p.name, p.is_fixed_role FROM sys.database_principals p
            WHERE name NOT IN (''db_owner'', ''public'', ''SaveToDB_Developers'', ''SaveToDB_Users'') AND p.[type] IN (''R'') AND p.is_fixed_role = 0
        ) AS t ORDER BY t.is_fixed_role, t.name
    FOR XML PATH('''')
    )

DECLARE @Query varchar(MAX)

SET @Query = ''
SELECT
    LOWER(p.[Type]) AS [Type]
    , p.[Name]
    , [SaveToDB_Developers], [SaveToDB_Users]'' + COALESCE(@List, '''') + ''
FROM
    (
    SELECT
        p.name AS [Role]
        , m.type_desc AS [Type]
        , m.name AS [Name]
        , 1 AS [include]
    FROM
        sys.database_principals m
        LEFT JOIN sys.database_role_members rm ON rm.member_principal_id = m.principal_id
        LEFT JOIN sys.database_principals p ON p.principal_id = rm.role_principal_id
    WHERE
        m.[type] IN (''''S'''', ''''U'''', ''''R'''')
        AND m.is_fixed_role = 0
        AND NOT m.name IN (''''dbo'''', ''''sys'''', ''''guest'''', ''''public'''', ''''INFORMATION_SCHEMA'''', ''''SaveToDB_Developers'''', ''''SaveToDB_Users'''')
    ) s PIVOT (
        SUM([include]) FOR [Role] IN ([SaveToDB_Developers], [SaveToDB_Users]''+ COALESCE(@List, '''') + '')
    ) p
ORDER BY
    p.[Type]
    , p.[Name]
''

EXEC(@Query)
', NULL, NULL, NULL, NULL)
GO
INSERT [dbo01].[QueryList] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_TYPE], [TABLE_CODE], [INSERT_PROCEDURE], [UPDATE_PROCEDURE], [DELETE_PROCEDURE], [PROCEDURE_TYPE]) VALUES (N'tools01', N'Permissions', N'CODE', N'WITH cte (object_schema, [object_name], object_type_desc,
    principal_name, principal_type_desc,
    permission_type_desc, state_desc,
    grantee_principal_id, sort_order)
AS 
    (
    SELECT
        [object_schema] = CASE p.class
            WHEN 4 THEN NULL
            WHEN 3 THEN SCHEMA_NAME(p.major_id)
            WHEN 1 THEN OBJECT_SCHEMA_NAME(p.major_id)
            WHEN 0 THEN NULL
            ELSE NULL
            END
        , [object_name] = CASE p.class
            WHEN 4 THEN USER_NAME(p.major_id)
            WHEN 3 THEN NULL
            WHEN 1 THEN OBJECT_NAME(p.major_id)
            WHEN 0 THEN DB_NAME()
            ELSE NULL
            END
        , [object_type_desc] = CASE
            WHEN p.class = 4 THEN dp.type_desc
            WHEN p.class = 3 THEN p.class_desc
            WHEN p.class = 1 AND p.major_id < 0 THEN so.type_desc
            WHEN p.class = 1 THEN o.type_desc
            WHEN p.class = 0 THEN p.class_desc
            ELSE p.class_desc
            END
        , r.name AS principal_name
        , r.type_desc AS principal_type_desc
        , p.[permission_name] AS permission_type_desc
        , p.state_desc
        , p.grantee_principal_id
        , sort_order = CASE
            WHEN p.class = 4 THEN 2
            WHEN p.class = 3 THEN 3
            WHEN p.class = 1 AND p.major_id < 0 THEN 10
            WHEN p.class = 1 THEN CASE o.type_desc
                WHEN ''SQL_STORED_PROCEDURE'' THEN 13
                WHEN ''SQL_SCALAR_FUNCTION'' THEN 14
                WHEN ''USER_TABLE'' THEN 11
                WHEN ''VIEW'' THEN 12
                ELSE 15
                END 
            WHEN p.class = 0 THEN 1
            ELSE 20
            END
    FROM
        sys.database_permissions p
        LEFT OUTER JOIN sys.objects o ON o.[object_id] = p.major_id AND p.class = 1
        LEFT OUTER JOIN sys.system_objects so ON so.[object_id] = p.major_id AND p.class = 1
        LEFT OUTER JOIN sys.database_principals dp ON dp.principal_id = p.major_id AND p.class = 4
        INNER JOIN sys.database_principals r ON r.principal_id = p.grantee_principal_id
    WHERE
        p.class IN (0, 1, 3, 4)
        AND NOT (so.[object_id] IS NULL AND p.major_id < 0 AND p.class = 1)
        AND NOT SCHEMA_NAME(p.major_id) IN (''sys'')
    )

SELECT
    principal_type_desc AS PrincipalType
    , principal_name AS Principal
    , object_schema AS [Schema]
    , [object_name] AS Name
    , object_type_desc AS [Type]
    , CASE WHEN state_mask >= 1000 THEN ''direct'' ELSE ''indirect'' END AS GrantType
    , [SELECT], [INSERT], [UPDATE], [DELETE]
    , [EXECUTE]
    , [VIEW DEFINITION]
    , [ALTER], [CONTROL], [REFERENCES], [TAKE OWNERSHIP]
    , [IMPERSONATE]
    , [CONNECT]
FROM
    (
        SELECT
            p.object_schema
            , p.[object_name]
            , object_type_desc = CASE MAX(p.object_type_desc)
                WHEN ''SQL_STORED_PROCEDURE'' THEN ''procedure''
                WHEN ''SQL_SCALAR_FUNCTION'' THEN ''function''
                WHEN ''USER_TABLE'' THEN ''table''
                WHEN ''VIEW'' THEN ''view''
                ELSE LOWER(MAX(p.object_type_desc))
                END 
            , p.principal_name
            , principal_type_desc = LOWER(MAX(p.principal_type_desc))
            , p.permission_type_desc
            , MAX(p.[state_mask]) AS state_mask
            , sort_order = CASE MAX(p.object_type_desc)
                WHEN ''SQL_STORED_PROCEDURE'' THEN 13
                WHEN ''SQL_SCALAR_FUNCTION'' THEN 14
                WHEN ''USER_TABLE'' THEN 11
                WHEN ''VIEW'' THEN 12
                ELSE MAX(p.sort_order)
                END 
            , state_desc = CASE MAX(p.[state_mask])
                WHEN 2000 THEN ''GRANT''
                WHEN 1000 THEN ''DENY''
                WHEN  200 THEN ''GRANT r''
                WHEN  100 THEN ''DENY  r''
                WHEN   20 THEN ''GRANT s''
                WHEN   10 THEN ''DENY  s''
                WHEN    2 THEN ''GRANT sr''
                WHEN    1 THEN ''DENY  sr''
                ELSE NULL
                END 
        FROM
            (
                SELECT
                    cte.object_schema
                    , cte.[object_name]
                    , cte.object_type_desc
                    , ''DIRECT'' AS [object_control]
                    , cte.principal_name
                    , cte.principal_type_desc
                    , ''DIRECT'' AS [principal_control]
                    , cte.permission_type_desc
                    , cte.state_desc
                    , cte.sort_order
                    , state_mask = CASE cte.state_desc WHEN ''DENY'' THEN 1000 ELSE 2000 END
                FROM
                    cte

                UNION ALL
                SELECT
                    cte.object_schema
                    , cte.[object_name]
                    , MAX(cte.object_type_desc) AS object_type_desc
                    , ''DIRECT'' AS [object_control]
                    , u.name AS principal_name
                    , MAX(u.type_desc) AS principal_type_desc
                    , ''BY ROLE'' AS [principal_control]
                    , cte.permission_type_desc
                    , cte.state_desc
                    , MAX(cte.sort_order) AS sort_order
                    , state_mask = CASE cte.state_desc WHEN ''DENY'' THEN 100 ELSE 200 END
                FROM
                    cte
                    INNER JOIN sys.database_role_members rm ON rm.role_principal_id = cte.grantee_principal_id
                    LEFT OUTER JOIN sys.database_principals u ON u.principal_id = rm.member_principal_id
                GROUP BY
                    object_schema
                    , [object_name]
                    , u.name
                    , permission_type_desc
                    , state_desc

                UNION ALL
                SELECT
                    cte.object_schema
                    , o.name AS [object_name]
                    , o.type_desc AS object_type_desc
                    , ''BY SCHEMA'' AS [object_control]
                    , cte.principal_name
                    , cte.principal_type_desc
                    , ''DIRECT'' AS [principal_control]
                    , cte.permission_type_desc
                    , cte.state_desc
                    , cte.sort_order
                    , state_mask = CASE cte.state_desc WHEN ''DENY'' THEN 10 ELSE 20 END
                FROM
                    cte
                    INNER JOIN sys.objects o ON OBJECT_SCHEMA_NAME(o.[object_id]) = cte.object_schema
                WHERE
                    o.[type] IN (''U'', ''V'', ''P'')
                    AND cte.[object_name] IS NULL
                    AND ((o.[type] IN (''U'', ''V'') AND cte.permission_type_desc NOT IN (''EXECUTE''))
                        OR (o.[type] = ''P'' AND cte.permission_type_desc NOT IN (''SELECT'', ''INSERT'', ''UPDATE'', ''DELETE'')))
        
                UNION ALL
                SELECT
                    cte.object_schema
                    , o.name AS [object_name]
                    , MAX(o.type_desc) AS object_type_desc
                    , ''BY SCHEMA'' AS [object_control]
                    , u.name AS principal_name
                    , MAX(u.type_desc) AS principal_type_desc
                    , ''BY ROLE'' AS [principal_control]
                    , cte.permission_type_desc
                    , cte.state_desc
                    , MAX(cte.sort_order) AS sort_order
                    , state_mask = CASE cte.state_desc WHEN ''DENY'' THEN 1 ELSE 2 END
                FROM
                    cte
                    INNER JOIN sys.objects o ON OBJECT_SCHEMA_NAME(o.[object_id]) = cte.object_schema
                    INNER JOIN sys.database_role_members rm ON rm.role_principal_id = cte.grantee_principal_id
                    LEFT OUTER JOIN sys.database_principals u ON u.principal_id = rm.member_principal_id
                WHERE
                    o.[type] IN (''U'', ''V'', ''P'')
                    AND cte.[object_name] IS NULL
                    AND ((o.[type] IN (''U'', ''V'') AND cte.permission_type_desc NOT IN (''EXECUTE''))
                        OR (o.[type] = ''P'' AND cte.permission_type_desc NOT IN (''SELECT'', ''INSERT'', ''UPDATE'', ''DELETE'')))
                GROUP BY
                    object_schema
                    , o.name
                    , u.name
                    , permission_type_desc
                    , state_desc
        ) p
        GROUP BY
            p.object_schema
            , p.[object_name]
            , p.principal_name
            , p.permission_type_desc
    ) s PIVOT (
        MAX(state_desc)
        FOR permission_type_desc IN (
            [SELECT], [INSERT], [UPDATE], [DELETE]
            , [EXECUTE]
            , [VIEW DEFINITION]
            , [ALTER], [CONTROL], [REFERENCES], [TAKE OWNERSHIP]
            , [IMPERSONATE]
            , [CONNECT]
            )
    ) p
WHERE
    COALESCE(p.object_schema, '''') = COALESCE(@Schema, p.object_schema, '''')
    AND COALESCE(p.object_type_desc, '''') = COALESCE(@Type, p.object_type_desc, '''')
    AND COALESCE(p.principal_name, '''') = COALESCE(@Principal, p.principal_name, '''')
    AND CASE WHEN state_mask >= 1000 THEN ''direct'' ELSE ''indirect'' END = COALESCE(@GrantType, CASE WHEN state_mask >= 1000 THEN ''direct'' ELSE ''indirect'' END, '''')
ORDER BY
    p.principal_type_desc
    , p.principal_name
    , p.object_schema
    , p.sort_order
    , p.object_type_desc
    , p.[object_name]
', NULL, NULL, NULL, NULL)
GO

INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'DatabaseObjects', N'Name', N'Change', N'tools01', N'DatabaseObjects_Name_Change', N'CODE', N'IF @ColumnName = ''Name''
    BEGIN
    EXEC sp_rename @ObjectName, @Name
    
    UPDATE dbo01.ObjectTranslation
    SET
        TABLE_NAME = @Name
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.ColumnTranslation
    SET
        TABLE_NAME = @Name
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.EventHandlers
    SET
        TABLE_NAME = @Name
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.ParameterValues
    SET
        TABLE_NAME = @Name
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.TableFormats
    SET
        TABLE_NAME = @Name
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName
    END
', N'_Reload', NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'DatabaseObjects', N'TranslatedName', N'Change', N'tools01', N'DatabaseObjects_TranslatedName_Change', N'CODE', N'IF @ColumnName = ''TranslatedName''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_NAME = @TranslatedName
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedDescription''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_DESC = @TranslatedDescription
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedComment''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_COMMENT = @TranslatedComment
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

IF @@ROWCOUNT = 0 AND @CellValue IS NOT NULL

    INSERT INTO dbo01.ObjectTranslation
        (TABLE_SCHEMA, TABLE_NAME, LANGUAGE_NAME, TRANSLATED_NAME, TRANSLATED_DESC, TRANSLATED_COMMENT)
    VALUES
        (@Schema, @Name, @Language, @TranslatedName, @TranslatedDescription, @TranslatedComment)

', NULL, NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'DatabaseObjects', N'TranslatedDesc', N'Change', N'tools01', N'DatabaseObjects_TranslatedDesc_Change', N'CODE', N'IF @ColumnName = ''TranslatedName''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_NAME = @TranslatedName
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedDescription''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_DESC = @TranslatedDescription
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedComment''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_COMMENT = @TranslatedComment
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

IF @@ROWCOUNT = 0 AND @CellValue IS NOT NULL

    INSERT INTO dbo01.ObjectTranslation
        (TABLE_SCHEMA, TABLE_NAME, LANGUAGE_NAME, TRANSLATED_NAME, TRANSLATED_DESC, TRANSLATED_COMMENT)
    VALUES
        (@Schema, @Name, @Language, @TranslatedName, @TranslatedDescription, @TranslatedComment)

', NULL, NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'DatabaseObjects', N'TranslatedComment', N'Change', N'tools01', N'DatabaseObjects_TranslatedComment_Change', N'CODE', N'IF @ColumnName = ''TranslatedName''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_NAME = @TranslatedName
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedDescription''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_DESC = @TranslatedDescription
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

ELSE IF @ColumnName = ''TranslatedComment''

    UPDATE dbo01.ObjectTranslation
    SET
        TRANSLATED_COMMENT = @TranslatedComment
    WHERE
        TABLE_SCHEMA = @Schema AND TABLE_NAME = @Name AND LANGUAGE_NAME = @Language

IF @@ROWCOUNT = 0 AND @CellValue IS NOT NULL

    INSERT INTO dbo01.ObjectTranslation
        (TABLE_SCHEMA, TABLE_NAME, LANGUAGE_NAME, TRANSLATED_NAME, TRANSLATED_DESC, TRANSLATED_COMMENT)
    VALUES
        (@Schema, @Name, @Language, @TranslatedName, @TranslatedDescription, @TranslatedComment)

', NULL, NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'DatabaseObjects', N'Schema', N'Change', N'tools01', N'DatabaseObjects_Schema_Change', N'CODE', N'IF @ColumnName = ''Schema''
    BEGIN
    EXECUTE (''ALTER SCHEMA ['' + @Schema + ''] TRANSFER '' + @ObjectName)

    UPDATE dbo01.ObjectTranslation
    SET
        TABLE_SCHEMA = @Schema
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.ColumnTranslation
    SET
        TABLE_SCHEMA = @Schema
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.EventHandlers
    SET
        TABLE_SCHEMA = @Schema
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.ParameterValues
    SET
        TABLE_SCHEMA = @Schema
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName

    UPDATE dbo01.TableFormats
    SET
        TABLE_SCHEMA = @Schema
    WHERE
        QUOTENAME(TABLE_SCHEMA) + ''.'' + QUOTENAME(TABLE_NAME) = @ObjectName
    END
', N'_Reload', NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'RoleMembers', NULL, N'Change', N'tools01', N'RoleMembers_Change', N'CODE', N'IF DATABASE_PRINCIPAL_ID(@ColumnName) IS NULL
    BEGIN
    RAISERROR(N''The role %s is not exists'', 10, 1, @ColumnName)
    RETURN
    END

IF DATABASE_PRINCIPAL_ID(@Name) IS NULL
    BEGIN
    RAISERROR(N''The user %s is not exists'', 10, 2, @User)
    RETURN
    END

IF @CellValue IS NULL
    EXEC sp_droprolemember @ColumnName, @Name
ELSE
    EXEC sp_addrolemember @ColumnName, @Name
', NULL, NULL, NULL, 1)
GO
INSERT [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'tools01', N'Permissions', NULL, N'Change', NULL, N'Permissions_Change', N'CODE', N'SET NOCOUNT ON

DECLARE @Permission varchar(128)
DECLARE @State varchar(128)

SET @Permission = UPPER(@ColumnName)
SET @State = UPPER(@CellValue)

IF CHARINDEX(''-'' + @Permission + ''-'',
    ''-SELECT-INSERT-UPDATE-DELETE-EXECUTE-VIEW DEFINITION-ALTER-CONTROL-IMPERSONATE-REFERENCES-TAKE OWNERSHIP-CONNECT-'') = 0
    RETURN

IF @State IS NULL
    SET @State = ''REVOKE''
ELSE IF @State = ''GRANT'' OR @State = ''G''
    SET @State = ''GRANT''
ELSE IF @State = ''DENY'' OR @State = ''D''
    SET @State = ''DENY''
ELSE IF @State = ''REVOKE'' OR @State = ''R''
    SET @State = ''REVOKE''
ELSE
    RETURN
        
DECLARE @Query varchar(max)

IF @Schema IS NULL
    BEGIN
    IF UPPER(@object_type_desc) = ''DATABASE''
        SET @Query = @State + '' '' + @Permission + '' TO '' + @Principal
    ELSE IF UPPER(@object_type_desc) = ''SQL_USER'' OR UPPER(@object_type_desc) = ''WINDOWS_USER''
        SET @Query = @State + '' '' + @Permission + '' ON USER::'' + QUOTENAME(@Name) + '' TO '' + @Principal
    ELSE IF UPPER(@object_type_desc) = ''DATABASE_ROLE'' OR UPPER(@object_type_desc) = ''APPLICATION_ROLE''
        SET @Query = @State + '' '' + @Permission + '' ON ROLE::'' + QUOTENAME(@Name) + '' TO '' + @Principal
    ELSE
        RETURN
    END
ELSE IF @Name IS NULL
    SET @Query = @State + '' '' + @Permission + '' ON SCHEMA::'' + QUOTENAME(@Schema) + '' TO '' + @Principal
ELSE
    SET @Query = @State + '' '' + @Permission + '' ON OBJECT::'' + 
        QUOTENAME(@Schema) + ''.'' + QUOTENAME(@Name) + '' TO '' + @Principal

EXEC (@Query)
', N'_Reload', NULL, NULL, 1)
GO

INSERT [dbo01].[ParameterValues] ([TABLE_SCHEMA], [TABLE_NAME], [PARAMETER_NAME], [SELECT_SCHEMA], [SELECT_NAME], [SELECT_TYPE], [SELECT_CODE]) VALUES (N'tools01', N'Permissions', N'Schema', N'tools01', N'Permissions_Schema', N'CODE', N'SELECT NULL AS [Schema] UNION ALL SELECT DISTINCT SCHEMA_NAME(p.major_id) FROM sys.database_permissions p WHERE p.class = 3')
INSERT [dbo01].[ParameterValues] ([TABLE_SCHEMA], [TABLE_NAME], [PARAMETER_NAME], [SELECT_SCHEMA], [SELECT_NAME], [SELECT_TYPE], [SELECT_CODE]) VALUES (N'tools01', N'Permissions', N'Principal', N'tools01', N'Permissions_Principal', N'CODE', N'SELECT NULL AS [Schema] UNION ALL SELECT name FROM sys.database_principals r WHERE r.is_fixed_role = 0 AND NOT r.name IN (''dbo'', ''sys'', ''public'', ''guest'', ''INFORMATION_SCHEMA'')')
INSERT [dbo01].[ParameterValues] ([TABLE_SCHEMA], [TABLE_NAME], [PARAMETER_NAME], [SELECT_SCHEMA], [SELECT_NAME], [SELECT_TYPE], [SELECT_CODE]) VALUES (N'tools01', N'Permissions', N'Type', N'tools01', N'Permissions_Type', N'CODE', N'SELECT NULL AS [Type] UNION SELECT ''schema'' UNION SELECT ''table'' UNION SELECT ''view''  UNION SELECT ''procedure'' UNION SELECT ''function'' UNION SELECT ''database_role'' UNION SELECT ''database_role'' UNION SELECT ''sql_user'' UNION SELECT ''windows_user''')
INSERT [dbo01].[ParameterValues] ([TABLE_SCHEMA], [TABLE_NAME], [PARAMETER_NAME], [SELECT_SCHEMA], [SELECT_NAME], [SELECT_TYPE], [SELECT_CODE]) VALUES (N'tools01', N'Permissions', N'GrantType', N'tools01', N'Permissions_GrantType', N'CODE', N'SELECT NULL AS GrantType UNION ALL SELECT ''direct''')

INSERT [dbo01].[TableFormats] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_EXCEL_FORMAT_XML]) VALUES (N'tools01', N'DatabaseObjects', N'<table name="tools01.DatabaseObjects"><columnFormats><column name="" property="ListObjectName" value="DatabaseObjects" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$3" type="String" /><column name="ID" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ID" property="Address" value="$C$3" type="String" /><column name="ObjectName" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="ObjectName" property="Address" value="$D$3" type="String" /><column name="Schema" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Schema" property="Address" value="$E$3" type="String" /><column name="Schema" property="ColumnWidth" value="9.43" type="Double" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name" property="Address" value="$F$3" type="String" /><column name="Name" property="ColumnWidth" value="57.57" type="Double" /><column name="Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Type" property="Address" value="$G$3" type="String" /><column name="Type" property="ColumnWidth" value="9.43" type="Double" /><column name="TranslatedName" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TranslatedName" property="Address" value="$H$3" type="String" /><column name="TranslatedName" property="ColumnWidth" value="30" type="Double" /><column name="TranslatedDescription" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TranslatedDescription" property="Address" value="$I$3" type="String" /><column name="TranslatedDescription" property="ColumnWidth" value="43" type="Double" /><column name="TranslatedComment" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TranslatedComment" property="Address" value="$J$3" type="String" /><column name="TranslatedComment" property="ColumnWidth" value="41.43" type="Double" /><column name="Alterable" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Alterable" property="Address" value="$K$3" type="String" /><column name="Alterable" property="ColumnWidth" value="10.86" type="Double" /><column name="CreateDate" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CreateDate" property="Address" value="$L$3" type="String" /><column name="CreateDate" property="ColumnWidth" value="12.57" type="Double" /><column name="CreateDate" property="NumberFormat" value="m/d/yyyy" type="String" /><column name="ModifyDate" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ModifyDate" property="Address" value="$M$3" type="String" /><column name="ModifyDate" property="ColumnWidth" value="13.14" type="Double" /><column name="ModifyDate" property="NumberFormat" value="m/d/yyyy" type="String" /><column name="SortFields(1)" property="KeyfieldName" value="Schema" type="String" /><column name="SortFields(1)" property="SortOn" value="0" type="Double" /><column name="SortFields(1)" property="Order" value="1" type="Double" /><column name="SortFields(1)" property="DataOption" value="2" type="Double" /><column name="SortFields(2)" property="KeyfieldName" value="Name" type="String" /><column name="SortFields(2)" property="SortOn" value="0" type="Double" /><column name="SortFields(2)" property="Order" value="1" type="Double" /><column name="SortFields(2)" property="DataOption" value="2" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /></columnFormats></table>')
INSERT [dbo01].[TableFormats] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_EXCEL_FORMAT_XML]) VALUES (N'tools01', N'RoleMembers', N'<table name="tools01.RoleMembers"><columnFormats><column name="" property="ListObjectName" value="RoleMembers" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Type" property="Address" value="$C$4" type="String" /><column name="Type" property="ColumnWidth" value="13" type="Double" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name" property="Address" value="$D$4" type="String" /><column name="Name" property="ColumnWidth" value="22.57" type="Double" /><column name="SaveToDB_Developers" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SaveToDB_Developers" property="Address" value="$E$4" type="String" /><column name="SaveToDB_Developers" property="ColumnWidth" value="22.71" type="Double" /><column name="SaveToDB_Users" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SaveToDB_Users" property="Address" value="$F$4" type="String" /><column name="SaveToDB_Users" property="ColumnWidth" value="17.43" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /></columnFormats></table>')
INSERT [dbo01].[TableFormats] ([TABLE_SCHEMA], [TABLE_NAME], [TABLE_EXCEL_FORMAT_XML]) VALUES (N'tools01', N'Permissions', N'<table name="tools01.Permissions"><columnFormats><column name="" property="ListObjectName" value="Permissions" type="String" /><column name="" property="ShowTotals" value="False" type="Boolean" /><column name="" property="TableStyle.Name" value="TableStyleMedium15" type="String" /><column name="" property="ShowTableStyleColumnStripes" value="False" type="Boolean" /><column name="" property="ShowTableStyleFirstColumn" value="False" type="Boolean" /><column name="" property="ShowShowTableStyleLastColumn" value="False" type="Boolean" /><column name="" property="ShowTableStyleRowStripes" value="False" type="Boolean" /><column name="_RowNum" property="EntireColumn.Hidden" value="True" type="Boolean" /><column name="_RowNum" property="Address" value="$B$4" type="String" /><column name="PrincipalType" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="PrincipalType" property="Address" value="$C$4" type="String" /><column name="PrincipalType" property="ColumnWidth" value="14.71" type="Double" /><column name="Principal" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Principal" property="Address" value="$D$4" type="String" /><column name="Principal" property="ColumnWidth" value="22.57" type="Double" /><column name="Schema" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Schema" property="Address" value="$E$4" type="String" /><column name="Schema" property="ColumnWidth" value="9.43" type="Double" /><column name="Name" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Name" property="Address" value="$F$4" type="String" /><column name="Name" property="ColumnWidth" value="41.57" type="Double" /><column name="Type" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="Type" property="Address" value="$G$4" type="String" /><column name="Type" property="ColumnWidth" value="9.43" type="Double" /><column name="GrantType" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="GrantType" property="Address" value="$H$4" type="String" /><column name="GrantType" property="ColumnWidth" value="11.86" type="Double" /><column name="SELECT" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="SELECT" property="Address" value="$I$4" type="String" /><column name="SELECT" property="ColumnWidth" value="8.71" type="Double" /><column name="INSERT" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="INSERT" property="Address" value="$J$4" type="String" /><column name="INSERT" property="ColumnWidth" value="8.71" type="Double" /><column name="UPDATE" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="UPDATE" property="Address" value="$K$4" type="String" /><column name="UPDATE" property="ColumnWidth" value="9.71" type="Double" /><column name="DELETE" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="DELETE" property="Address" value="$L$4" type="String" /><column name="DELETE" property="ColumnWidth" value="8.71" type="Double" /><column name="EXECUTE" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="EXECUTE" property="Address" value="$M$4" type="String" /><column name="EXECUTE" property="ColumnWidth" value="10.29" type="Double" /><column name="VIEW DEFINITION" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="VIEW DEFINITION" property="Address" value="$N$4" type="String" /><column name="VIEW DEFINITION" property="ColumnWidth" value="18.29" type="Double" /><column name="ALTER" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="ALTER" property="Address" value="$O$4" type="String" /><column name="ALTER" property="ColumnWidth" value="7.86" type="Double" /><column name="CONTROL" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CONTROL" property="Address" value="$P$4" type="String" /><column name="CONTROL" property="ColumnWidth" value="11" type="Double" /><column name="REFERENCES" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="REFERENCES" property="Address" value="$Q$4" type="String" /><column name="REFERENCES" property="ColumnWidth" value="13.43" type="Double" /><column name="TAKE OWNERSHIP" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="TAKE OWNERSHIP" property="Address" value="$R$4" type="String" /><column name="TAKE OWNERSHIP" property="ColumnWidth" value="18.57" type="Double" /><column name="IMPERSONATE" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="IMPERSONATE" property="Address" value="$S$4" type="String" /><column name="IMPERSONATE" property="ColumnWidth" value="15.57" type="Double" /><column name="CONNECT" property="EntireColumn.Hidden" value="False" type="Boolean" /><column name="CONNECT" property="Address" value="$T$4" type="String" /><column name="CONNECT" property="ColumnWidth" value="11.14" type="Double" /><column name="SELECT" property="FormatConditions(1).ColumnsCount" value="12" type="Double" /><column name="SELECT" property="FormatConditions(1).AppliesTo.Address" value="$I$4:$T$951" type="String" /><column name="SELECT" property="FormatConditions(1).Type" value="9" type="Double" /><column name="SELECT" property="FormatConditions(1).Priority" value="5" type="Double" /><column name="SELECT" property="FormatConditions(1).StopIfTrue" value="True" type="Boolean" /><column name="SELECT" property="FormatConditions(1).Formula1" value="=NOT(ISERROR(SEARCH(&quot;DENY &quot;;I4)))" type="String" /><column name="SELECT" property="FormatConditions(1).Text" value="DENY " type="String" /><column name="SELECT" property="FormatConditions(1).Font.Color" value="393372" type="Double" /><column name="SELECT" property="FormatConditions(1).Interior.Color" value="13551615" type="Double" /><column name="SELECT" property="FormatConditions(1).Interior.PatternThemeColor" value="-1" type="Double" /><column name="SELECT" property="FormatConditions(2).ColumnsCount" value="12" type="Double" /><column name="SELECT" property="FormatConditions(2).AppliesTo.Address" value="$I$4:$T$951" type="String" /><column name="SELECT" property="FormatConditions(2).Type" value="9" type="Double" /><column name="SELECT" property="FormatConditions(2).Priority" value="6" type="Double" /><column name="SELECT" property="FormatConditions(2).StopIfTrue" value="True" type="Boolean" /><column name="SELECT" property="FormatConditions(2).Formula1" value="=NOT(ISERROR(SEARCH(&quot;GRANT &quot;;I4)))" type="String" /><column name="SELECT" property="FormatConditions(2).Text" value="GRANT " type="String" /><column name="SELECT" property="FormatConditions(2).Font.Color" value="24832" type="Double" /><column name="SELECT" property="FormatConditions(2).Interior.Color" value="13561798" type="Double" /><column name="SELECT" property="FormatConditions(2).Interior.PatternThemeColor" value="-1" type="Double" /><column name="SELECT" property="FormatConditions(3).ColumnsCount" value="12" type="Double" /><column name="SELECT" property="FormatConditions(3).AppliesTo.Address" value="$I$4:$T$951" type="String" /><column name="SELECT" property="FormatConditions(3).Type" value="1" type="Double" /><column name="SELECT" property="FormatConditions(3).Priority" value="7" type="Double" /><column name="SELECT" property="FormatConditions(3).StopIfTrue" value="True" type="Boolean" /><column name="SELECT" property="FormatConditions(3).Formula1" value="=&quot;DENY&quot;" type="String" /><column name="SELECT" property="FormatConditions(3).Operator" value="3" type="Double" /><column name="SELECT" property="FormatConditions(3).Interior.Color" value="255" type="Double" /><column name="SELECT" property="FormatConditions(3).Interior.PatternThemeColor" value="-1" type="Double" /><column name="SELECT" property="FormatConditions(4).ColumnsCount" value="12" type="Double" /><column name="SELECT" property="FormatConditions(4).AppliesTo.Address" value="$I$4:$T$951" type="String" /><column name="SELECT" property="FormatConditions(4).Type" value="1" type="Double" /><column name="SELECT" property="FormatConditions(4).Priority" value="8" type="Double" /><column name="SELECT" property="FormatConditions(4).StopIfTrue" value="True" type="Boolean" /><column name="SELECT" property="FormatConditions(4).Formula1" value="=&quot;GRANT&quot;" type="String" /><column name="SELECT" property="FormatConditions(4).Operator" value="3" type="Double" /><column name="SELECT" property="FormatConditions(4).Interior.Color" value="5287936" type="Double" /><column name="SELECT" property="FormatConditions(4).Interior.PatternThemeColor" value="-1" type="Double" /><column name="" property="ActiveWindow.DisplayGridlines" value="False" type="Boolean" /><column name="" property="ActiveWindow.FreezePanes" value="True" type="Boolean" /><column name="" property="ActiveWindow.Split" value="True" type="Boolean" /><column name="" property="ActiveWindow.SplitRow" value="0" type="Double" /><column name="" property="ActiveWindow.SplitColumn" value="-2" type="Double" /><column name="" property="PageSetup.Orientation" value="2" type="Double" /><column name="" property="PageSetup.FitToPagesWide" value="1" type="Double" /><column name="" property="PageSetup.FitToPagesTall" value="1" type="Double" /></columnFormats></table>')

INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'QueryList', NULL, N'Actions', NULL, N'MenuSeparator', N'MENUSEPARATOR', NULL, NULL, 10, NULL, 1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'QueryList', NULL, N'Actions', N'tools01', N'Update Types', N'CODE', N'UPDATE dbo01.QueryList
SET
    TABLE_SCHEMA = t.TABLE_SCHEMA
    , TABLE_NAME = t.TABLE_NAME
    , TABLE_TYPE = t.TABLE_TYPE
    , TABLE_CODE = t.TABLE_CODE
    , INSERT_PROCEDURE = t.INSERT_PROCEDURE
    , UPDATE_PROCEDURE = t.UPDATE_PROCEDURE
    , DELETE_PROCEDURE = t.DELETE_PROCEDURE
    , PROCEDURE_TYPE = t.PROCEDURE_TYPE
FROM
    dbo01.QueryList c
    INNER JOIN (
        SELECT
            c.ID
            , COALESCE(t.TABLE_SCHEMA, r.ROUTINE_SCHEMA, c.TABLE_SCHEMA) AS TABLE_SCHEMA
            , COALESCE(t.TABLE_NAME, r.ROUTINE_NAME, c.TABLE_NAME) AS TABLE_NAME
            , COALESCE(t.TABLE_TYPE, r.ROUTINE_TYPE, c.TABLE_TYPE) AS TABLE_TYPE
            , c.TABLE_CODE
            , COALESCE(t1.TABLE_SCHEMA + ''.'' + t1.TABLE_NAME, r1.ROUTINE_SCHEMA + ''.'' + r1.ROUTINE_NAME, c.INSERT_PROCEDURE) AS INSERT_PROCEDURE
            , COALESCE(t2.TABLE_SCHEMA + ''.'' + t2.TABLE_NAME, r2.ROUTINE_SCHEMA + ''.'' + r2.ROUTINE_NAME, c.UPDATE_PROCEDURE) AS UPDATE_PROCEDURE
            , COALESCE(t3.TABLE_SCHEMA + ''.'' + t3.TABLE_NAME, r3.ROUTINE_SCHEMA + ''.'' + r3.ROUTINE_NAME, c.DELETE_PROCEDURE) AS DELETE_PROCEDURE
            , CASE
                WHEN c.INSERT_PROCEDURE IS NULL THEN NULL
                WHEN r3.ROUTINE_NAME IS NOT NULL THEN ''PROCEDURE''
                WHEN t3.TABLE_NAME IS NOT NULL THEN ''TABLE''
                WHEN c.DELETE_PROCEDURE IS NOT NULL THEN ''CODE''
                WHEN r1.ROUTINE_NAME IS NOT NULL THEN ''PROCEDURE''
                WHEN t1.TABLE_NAME IS NOT NULL THEN ''MERGE''
                ELSE ''CODE''
                END AS PROCEDURE_TYPE
        FROM
            dbo01.QueryList c
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t1 ON UPPER(t1.TABLE_SCHEMA + ''.'' + t1.TABLE_NAME) = UPPER(c.INSERT_PROCEDURE)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t2 ON UPPER(t2.TABLE_SCHEMA + ''.'' + t2.TABLE_NAME) = UPPER(c.UPDATE_PROCEDURE)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t3 ON UPPER(t3.TABLE_SCHEMA + ''.'' + t3.TABLE_NAME) = UPPER(c.DELETE_PROCEDURE)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r1 ON UPPER(r1.ROUTINE_SCHEMA + ''.'' + r1.ROUTINE_NAME) = UPPER(c.INSERT_PROCEDURE)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r2 ON UPPER(r2.ROUTINE_SCHEMA + ''.'' + r2.ROUTINE_NAME) = UPPER(c.UPDATE_PROCEDURE)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r3 ON UPPER(r3.ROUTINE_SCHEMA + ''.'' + r3.ROUTINE_NAME) = UPPER(c.DELETE_PROCEDURE)
    ) t ON t.ID = c.ID
', N'_Reload', 1, NULL, 1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'EventHandlers', NULL, N'Actions', NULL, N'MenuSeparator', N'MENUSEPARATOR', NULL, NULL, 10, NULL, 1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'EventHandlers', NULL, N'Actions', N'tools01', N'Update Types', N'CODE', N'UPDATE dbo01.EventHandlers
SET
    TABLE_SCHEMA = t.TABLE_SCHEMA
    , TABLE_NAME = t.TABLE_NAME
    , COLUMN_NAME = t.COLUMN_NAME
    , EVENT_NAME = t.EVENT_NAME
    , HANDLER_SCHEMA = t.HANDLER_SCHEMA
    , HANDLER_NAME = t.HANDLER_NAME
    , HANDLER_TYPE = t.HANDLER_TYPE
    , HANDLER_CODE = t.HANDLER_CODE
    , TARGET_WORKSHEET = t.TARGET_WORKSHEET
    , MENU_ORDER = t.MENU_ORDER
    , EDIT_PARAMETERS = t.EDIT_PARAMETERS
    , IS_ACTIVE = t.IS_ACTIVE
FROM
    dbo01.EventHandlers c
    INNER JOIN (
        SELECT
            c.ID
            , COALESCE(t.TABLE_SCHEMA, r.ROUTINE_SCHEMA, c.TABLE_SCHEMA) AS TABLE_SCHEMA
            , COALESCE(t.TABLE_NAME, r.ROUTINE_NAME, c.TABLE_NAME) AS TABLE_NAME
            , CASE UPPER(c.COLUMN_NAME)
                WHEN ''ACTIONS'' THEN ''Actions''
                WHEN ''CHANGE'' THEN ''Change''
                WHEN ''CONTEXTMENU'' THEN ''ContextMenu''
                WHEN ''DOUBLECLICK'' THEN ''DoubleClick''
                WHEN ''SELECTIONCHANGE'' THEN ''SelectionChange''
                ELSE c.COLUMN_NAME
                END AS COLUMN_NAME
            , c.EVENT_NAME
            , COALESCE(t1.TABLE_SCHEMA, r1.ROUTINE_SCHEMA, t2.TABLE_SCHEMA, r2.ROUTINE_SCHEMA, c.HANDLER_SCHEMA, CASE
                WHEN c.HANDLER_TYPE IN (''HTTP'', ''TEXT'', ''CODE'', ''MACRO'', ''CMD'') THEN LOWER(c.HANDLER_TYPE)
                WHEN CHARINDEX(''http://'', c.HANDLER_CODE) = 1 THEN ''http''
                WHEN CHARINDEX(''https://'', c.HANDLER_CODE) = 1 THEN ''http''
                ELSE NULL
                END) AS HANDLER_SCHEMA
            , COALESCE(t1.TABLE_NAME, r1.ROUTINE_NAME, t2.TABLE_NAME, r2.ROUTINE_NAME, c.HANDLER_NAME) AS HANDLER_NAME
            , COALESCE(t1.TABLE_TYPE, r1.ROUTINE_TYPE, t2.TABLE_TYPE, r2.ROUTINE_TYPE, c.HANDLER_TYPE, CASE
                WHEN CHARINDEX(''http://'', c.HANDLER_CODE) = 1 THEN ''HTTP''
                WHEN CHARINDEX(''https://'', c.HANDLER_CODE) = 1 THEN ''HTTP''
                WHEN UPPER(c.HANDLER_NAME) = ''MENUSEPARATOR'' THEN ''MENUSEPARATOR''
                ELSE NULL
                END) AS HANDLER_TYPE
            , c.HANDLER_CODE
            , c.TARGET_WORKSHEET
            , c.MENU_ORDER
            , c.EDIT_PARAMETERS
            , c.IS_ACTIVE
        FROM
            dbo01.EventHandlers c
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t1 ON UPPER(t1.TABLE_SCHEMA) = UPPER(c.HANDLER_SCHEMA) AND UPPER(t1.TABLE_NAME) = UPPER(c.HANDLER_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t2 ON UPPER(t2.TABLE_SCHEMA + ''.'' + t2.TABLE_NAME) = UPPER(c.HANDLER_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r1 ON UPPER(r1.ROUTINE_SCHEMA) = UPPER(c.HANDLER_SCHEMA) AND UPPER(r1.ROUTINE_NAME) = UPPER(c.HANDLER_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r2 ON UPPER(r2.ROUTINE_SCHEMA + ''.'' + r2.ROUTINE_NAME) = UPPER(c.HANDLER_NAME)
    ) t ON t.ID = c.ID
', N'_Reload', 1, NULL, 1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'ParameterValues', NULL, N'Actions', NULL, N'MenuSeparator', N'MENUSEPARATOR', NULL, NULL, 10, NULL, 1);
INSERT INTO [dbo01].[EventHandlers] ([TABLE_SCHEMA], [TABLE_NAME], [COLUMN_NAME], [EVENT_NAME], [HANDLER_SCHEMA], [HANDLER_NAME], [HANDLER_TYPE], [HANDLER_CODE], [TARGET_WORKSHEET], [MENU_ORDER], [EDIT_PARAMETERS], [IS_ACTIVE]) VALUES (N'dbo01', N'ParameterValues', NULL, N'Actions', N'tools01', N'Update Types', N'CODE', N'UPDATE dbo01.ParameterValues
SET
    TABLE_SCHEMA = t.TABLE_SCHEMA
    , TABLE_NAME = t.TABLE_NAME
    , PARAMETER_NAME = t.PARAMETER_NAME
    , SELECT_SCHEMA = t.SELECT_SCHEMA
    , SELECT_NAME = t.SELECT_NAME
    , SELECT_TYPE = t.SELECT_TYPE
    , SELECT_CODE = t.SELECT_CODE
FROM
    dbo01.ParameterValues c
    INNER JOIN (
        SELECT
            c.ID
            , COALESCE(t.TABLE_SCHEMA, r.ROUTINE_SCHEMA, c.TABLE_SCHEMA) AS TABLE_SCHEMA
            , COALESCE(t.TABLE_NAME, r.ROUTINE_NAME, c.TABLE_NAME) AS TABLE_NAME
            , c.PARAMETER_NAME
            , COALESCE(t1.TABLE_SCHEMA, r1.ROUTINE_SCHEMA, t2.TABLE_SCHEMA, r2.ROUTINE_SCHEMA, c.SELECT_SCHEMA, CASE
                WHEN c.SELECT_TYPE IN (''CODE'') THEN ''code''
                ELSE NULL
                END) AS SELECT_SCHEMA
            , COALESCE(t1.TABLE_NAME, r1.ROUTINE_NAME, t2.TABLE_NAME, r2.ROUTINE_NAME, c.SELECT_NAME) AS SELECT_NAME
            , COALESCE(t1.TABLE_TYPE, t2.TABLE_TYPE, r1.ROUTINE_TYPE, r2.ROUTINE_TYPE, c.SELECT_TYPE) AS SELECT_TYPE
            , c.SELECT_CODE
        FROM
            dbo01.ParameterValues c
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t ON UPPER(t.TABLE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(t.TABLE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r ON UPPER(r.ROUTINE_SCHEMA) = UPPER(c.TABLE_SCHEMA) AND UPPER(r.ROUTINE_NAME) = UPPER(c.TABLE_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t1 ON UPPER(t1.TABLE_SCHEMA) = UPPER(c.SELECT_SCHEMA) AND UPPER(t1.TABLE_NAME) = UPPER(c.SELECT_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.TABLES t2 ON UPPER(t2.TABLE_SCHEMA + ''.'' + t2.TABLE_NAME) = UPPER(c.SELECT_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r1 ON UPPER(r1.ROUTINE_SCHEMA) = UPPER(c.SELECT_SCHEMA) AND UPPER(r1.ROUTINE_NAME) = UPPER(c.SELECT_NAME)
            LEFT OUTER JOIN INFORMATION_SCHEMA.ROUTINES r2 ON UPPER(r2.ROUTINE_SCHEMA + ''.'' + r2.ROUTINE_NAME) = UPPER(c.SELECT_NAME)
    ) t ON t.ID = c.ID
', N'_Reload', 1, NULL, 1);

IF @@TRANCOUNT > 0
    COMMIT
